@@ -1,11 +1,9 @@
-/* $Id: Sybase.h,v 1.5 2001/08/03 20:05:59 mpeppler Exp $
+/* $Id: Sybase.h,v 1.8 2004/01/06 01:29:56 mpeppler Exp $
 
-   Copyright (c) 1997 - 2001 Michael Peppler
+   Copyright (c) 1997 - 2004 Michael Peppler
 
    You may distribute under the terms of either the GNU General Public
-   License or the Artistic License, as specified in the Perl README file,
-   with the exception that it cannot be placed on a CD-ROM or similar media
-   for commercial distribution without the prior approval of the author.
+   License or the Artistic License, as specified in the Perl README file.
 
 */
 
@@ -20,7 +18,7 @@
 /* These defines avoid name clashes for multiple statically linked DBD's    */
  
 #define dbd_init        syb_init
-#define dbd_db_login        syb_db_login
+#define dbd_db_login6       syb_db_login
 #define dbd_db_do       syb_db_do
 #define dbd_db_commit       syb_db_commit
 #define dbd_db_rollback     syb_db_rollback
@@ -79,7 +77,7 @@ void     syb_init _((dbistate_t *dbistate));
 
 int      syb_discon_all _((SV *drh, imp_drh_t *imp_drh));
 
-int      syb_db_login _((SV *dbh, imp_dbh_t *imp_dbh, char *dbname, char *uid, char *pwd));
+int      syb_db_login _((SV *dbh, imp_dbh_t *imp_dbh, char *dbname, char *uid, char *pwd, SV *attribs));
 int      syb_db_do _((SV *sv, char *statement));
 int      syb_db_commit     _((SV *dbh, imp_dbh_t *imp_dbh));
 int      syb_db_rollback   _((SV *dbh, imp_dbh_t *imp_dbh));
@@ -113,6 +111,14 @@ int      syb_bind_ph  _((SV *sth, imp_sth_t *imp_sth,
                 SV *param, SV *value, IV sql_type, SV *attribs,
 				int is_inout, IV maxlen));
 
+
+/* prototypes for module-specific functions */
+int      syb_thread_enabled _((void));
+int      syb_set_timeout _((int timeout));
+int      syb_db_date_fmt _((SV *, imp_dbh_t *, char *));
+
+
+
 #endif /* defined Sybase_h */
 
 /* end of Sybase.h */
